@@ -23,15 +23,15 @@ File-related utilities
 """
 
 
-import ConfigParser
 import datetime
 import glob
 import locale
 import os
-import os.path
-import shlex
+import re
 import stat
 import subprocess
+import sys
+import tempfile
 
 
 
@@ -75,34 +75,45 @@ class OpenFiles:
         return filename in self.files
 
 
-def bytes_to_human(bytes):
+def bytes_to_human(bytes_i):
     """Display a file size in human terms (megabytes, etc.)"""
 
     storage_multipliers = { 1024**5 : 'PB', 1024**4 : 'TB', \
         1024**3 : 'GB', 1024**2: 'MB', 1024: 'KB', 1 : 'B' }
 
-    if 0 == bytes:
+    assert(type(bytes_i) is int or type(bytes_i) is long)
+
+    if 0 == bytes_i:
         return "0"
 
-    if bytes >= 1024**3:
+    if bytes_i >= 1024**3:
         decimals = 2
-    elif bytes >= 1024:
+    elif bytes_i >= 1024:
         decimals = 1
     else:
         decimals = 0
 
     for key in sorted(storage_multipliers.keys(), reverse = True):
-        if bytes >= key:
-            abbrev = (1.0 * bytes) / key
+        if bytes_i >= key:
+            abbrev = (1.0 * bytes_i) / key
             suf = storage_multipliers[key]
-            format = "%." + str(decimals) + "f" + suf
+            strformat = "%." + str(decimals) + "f" + suf
             if hasattr(locale, 'format_string'):
-                return locale.format_string(format, abbrev)
+                return locale.format_string(strformat, abbrev)
             else:
-                return locale.format(format, abbrev)
+                return locale.format(strformat, abbrev)
+
+    if bytes_i < 0:
+        return "-" + bytes_to_human(abs(bytes_i))
+
 
 def children_in_directory(top, list_directories = False):
     """Iterate files and, optionally, subdirectories in directory"""
+    if type(top) is tuple:
+        for top_ in top:
+            for pathname in children_in_directory(top_, list_directories):
+                yield pathname
+        return
     for (dirpath, dirnames, filenames) in os.walk(top, topdown=False):
         if list_directories:
             for dirname in dirnames:
@@ -120,17 +131,13 @@ def delete(path, shred = False):
     elif stat.S_ISDIR(mode):
         os.rmdir(path)
     elif stat.S_ISREG(mode):
-        if options.get('shred') or shred:
-            # http://en.wikipedia.org/wiki/Data_remanence
-            # 2006 NIST Special Publication 800-88 (p. 7): "Studies have
-            # shown that most of today's media can be effectively cleared
-            # by one overwrite"
-            args = ["shred", "--remove", "--iterations=0", "--zero", path]
-            ret = subprocess.call(args)
-            if 0 != ret:
-                raise Exception("shred subprocess returned non-zero error code " % (ret,))
-        else:
-            os.remove(path)
+        if shred or options.get('shred'):
+            try:
+                wipe_contents(path)
+            except IOError, e:
+                # permission denied (13) happens shredding MSIE 8 on Windows 7
+                print "debug: IOError #%s shredding '%s'" % (e.errno, path)
+        os.remove(path)
     else:
         raise Exception("Unsupported special file type")
 
@@ -142,7 +149,10 @@ def ego_owner(filename):
 
 def exists_in_path(filename):
     """Returns boolean whether the filename exists in the path"""
-    for dirname in os.getenv('PATH').split(":"):
+    delimiter = ':'
+    if sys.platform == 'win32':
+        delimiter = ';'
+    for dirname in os.getenv('PATH').split(delimiter):
         if os.path.exists(os.path.join(dirname, filename)):
             return True
     return False
@@ -163,29 +173,170 @@ def execute_sqlite3(path, cmds):
     """Execute 'cmds' on SQLite database 'path'"""
     try:
         import sqlite3
-    except ImportError, e:
-        import sys
+    except ImportError, exc:
         if sys.version_info[0] == 2 and sys.version_info[1] < 5:
             raise RuntimeError(_("Cannot import Python module sqlite3: Python 2.5 or later is required."))
         else:
-            raise e
+            raise exc
     conn = sqlite3.connect(path)
     cursor = conn.cursor()
     for cmd in cmds.split(';'):
         try:
             cursor.execute(cmd)
-        except sqlite3.OperationalError, e:
-            raise sqlite3.OperationalError('%s: %s' % (e, path))
+        except sqlite3.OperationalError, exc:
+            if exc.message.find('no such function: ') >= 0:
+                # fixme: determine why randomblob and zeroblob are not available
+                print 'warning: %s' % exc.message
+            else:
+                raise sqlite3.OperationalError('%s: %s' % (exc, path))
     cursor.close()
     conn.commit()
     conn.close()
 
 
+def expand_glob_join(pathname1, pathname2):
+    """Join pathname1 and pathname1, expand pathname, glob, and return as list"""
+    ret = []
+    pathname3 = os.path.expanduser(os.path.expandvars(os.path.join(pathname1, pathname2)))
+    for pathname4 in glob.iglob(pathname3):
+        ret.append(pathname4)
+    return ret
+
+
 def getsize(path):
     """Return the actual file size considering spare files
        and symlinks"""
-    __stat = os.lstat(path)
-    return __stat.st_blocks * 512
+    if sys.platform == 'linux2':
+        __stat = os.lstat(path)
+        return __stat.st_blocks * 512
+    return os.path.getsize(path)
+
+
+def getsizedir(path):
+    """Return the size of the contents of a directory"""
+    total_bytes = 0
+    for node in children_in_directory(path, list_directories = False):
+        total_bytes += getsize(node)
+    return total_bytes
+
+
+def globex(pathname, regex):
+    """Yield a list of files with pathname and filter by regex"""
+    if type(pathname) is tuple:
+        for singleglob in pathname:
+            for path in globex(singleglob, regex):
+                yield path
+    else:
+        for path in glob.iglob(pathname):
+            if re.search(regex, path):
+                yield path
+
+
+def human_to_bytes(string):
+    """Convert a string like 10.2GB into bytes"""
+    multiplier = { 'B' : 1, 'KB': 1024, 'MB': 1024**2, \
+        'GB': 1024**3, 'TB': 1024**4 }
+    matches = re.findall("^([0-9]*)(\.[0-9]{1,2})?([KMGT]{0,1}B)$", string)
+    if 2 > len(matches[0]):
+        raise ValueError("Invalid input for '%s'" % (string))
+    return int(float(matches[0][0]+matches[0][1]) * multiplier[matches[0][2]])
+
+
+def listdir(directory):
+    """Return full path of files in directory.
+
+    Path may be a tuple of directories."""
+
+    if type(directory) is tuple:
+        for dirname in directory:
+            for pathname in listdir(dirname):
+                yield pathname
+        return
+    dirname = os.path.expanduser(directory)
+    if not os.path.lexists(dirname):
+        return
+    for filename in os.listdir(dirname):
+        yield os.path.join(dirname, filename)
+
+
+def wipe_contents(path):
+    """Wipe files contents
+
+    http://en.wikipedia.org/wiki/Data_remanence
+    2006 NIST Special Publication 800-88 (p. 7): "Studies have
+    shown that most of today's media can be effectively cleared
+    by one overwrite"
+    """
+    size = getsize(path)
+    try:
+        f = open(path, 'wb')
+    except IOError, e:
+        if 13 == e.errno: # permission denied
+            os.chmod(path, 0200) # user write only
+            f = open(path, 'wb')
+        else:
+            raise
+    blanks =  chr(0) * 4096
+    while size > 0:
+        f.write(blanks)
+        size -= 4096
+    f.flush()
+    f.truncate(0)
+    f.flush()
+    f.close()
+
+
+def wipe_path(pathname, idle_cb = None, ):
+    """Wipe the free space in the path"""
+    print "debug: wipe_path('%s')" % pathname
+    files = []
+    total_bytes = 0
+    # repeat to clear inodes (Linux) / MFT (Master File Table on Windows)
+    while True:
+        try:
+            f = tempfile.TemporaryFile(dir = pathname)
+        except OSError, e:
+            # Linux gives errno 24
+            # Windows gives errno 28 No space left on device
+            if e.errno in (24, 28):
+                break
+            else:
+                raise
+        last_idle = datetime.datetime.now()
+        # blocks
+        blanks = chr(0) * 4096
+        try:
+            while True:
+                f.write(blanks)
+                if idle_cb and (last_idle - datetime.datetime.now()).seconds > 1:
+                    # keep the GUI responding
+                    idle_cb()
+        except IOError, e:
+            if 28 != e.errno:
+                raise
+        # individual characters
+        try:
+            while True:
+                f.write(chr(0))
+        except IOError, e:
+            if 28 != e.errno:
+                raise
+        try:
+            f.flush()
+        except:
+            # IOError: [Errno 28] No space left on device
+            # seen on Microsoft Windows XP SP3 with ~30GB free space but
+            # not on another XP SP3 with 64MB free space
+            print "info: exception on f.flush()"
+        files.append(f)
+        total_bytes += f.tell()
+    # statistics
+    print 'debug: wrote %d files and %d bytes' % (len(files), total_bytes)
+    # truncate and close files
+    for f in files:
+        f.truncate(0)
+        f.close()
+    # files are removed implicitly
 
 
 def vacuum_sqlite3(path):
@@ -203,28 +354,19 @@ class TestFileUtilities(unittest.TestCase):
 
     def __touch(self, filename):
         """Create an empty file"""
-        f = open(filename, "w")
-
-
-    def __human_to_bytes(self, string):
-        """Convert a string like 10.2GB into bytes"""
-        multiplier = { 'B' : 1, 'KB': 1024, 'MB': 1024**2, \
-            'GB': 1024**3, 'TB': 1024**4 }
-        import re
-        matches = re.findall("^([0-9]*)(\.[0-9]{1,2})?([KMGT]{0,1}B)$", string)
-        if 2 > len(matches[0]):
-            raise ValueError("Invalid input for '%s'" % (string))
-        return int(float(matches[0][0]+matches[0][1]) * multiplier[matches[0][2]])
+        open(filename, "w")
 
 
     def test_bytes_to_human(self):
         """Unit test for class bytes_to_human"""
 
-        old_locale = locale.getlocale(locale.LC_NUMERIC)
-        locale.setlocale(locale.LC_NUMERIC, 'en_US.UTF-8')
+        if sys.platform == 'linux2':
+            old_locale = locale.getlocale(locale.LC_NUMERIC)
+            locale.setlocale(locale.LC_NUMERIC, 'en_US.UTF-8')
 
         # test one-way conversion for predefined values
-        tests = [ ("0", bytes_to_human(0)),
+        tests = [ ("-1B", bytes_to_human(-1)),
+                  ("0", bytes_to_human(0)),
                   ("1B", bytes_to_human(1)),
                   ("1.0KB", bytes_to_human(1024)),
                   ("1.0MB", bytes_to_human(1024**2)),
@@ -237,13 +379,13 @@ class TestFileUtilities(unittest.TestCase):
         # test roundtrip conversion for random values
         import random
         for n in range(0, 1000):
-            bytes = random.randrange(0, 1024**4)
-            human = bytes_to_human(bytes)
-            bytes2 = self.__human_to_bytes(human)
-            error =  abs(float(bytes2 - bytes) / bytes)
+            bytes1 = random.randrange(0, 1024**4)
+            human = bytes_to_human(bytes1)
+            bytes2 = human_to_bytes(human)
+            error =  abs(float(bytes2 - bytes1) / bytes1)
             self.assert_(abs(error) < 0.01, \
                 "%d (%s) is %.2f%% different than %d" % \
-                (bytes, human, error * 100, bytes2))
+                (bytes1, human, error * 100, bytes2))
 
         # test localization
         if hasattr(locale, 'format_string'):
@@ -255,12 +397,12 @@ class TestFileUtilities(unittest.TestCase):
                 self.assertEqual("1,00TB", bytes_to_human(1024**4))
 
         # clean up
-        locale.setlocale(locale.LC_NUMERIC, old_locale)
+        if sys.platform == 'linux2':
+            locale.setlocale(locale.LC_NUMERIC, old_locale)
 
 
     def test_children_in_directory(self): 
         """Unit test for function children_in_directory()"""
-        import tempfile
 
         # test an existing directory that usually exists
         dirname = os.path.expanduser("~/.config")
@@ -296,35 +438,55 @@ class TestFileUtilities(unittest.TestCase):
 
     def test_delete(self):
         """Unit test for method delete()"""
+        print "testing delete() with shred = False"
+        self.delete_helper(shred = False)
+        print "testing delete() with shred = True"
+        self.delete_helper(shred = True)
+
+
+    def delete_helper(self, shred):
+        """Called by test_delete() with shred = False and = True"""
 
-        import tempfile
 
         hebrew = "עִבְרִית"
         katanana = "アメリカ"
         umlauts = "ÄäǞǟËëḦḧÏïḮḯÖöȪȫṎṏT̈ẗÜüǕǖǗǘǙǚǛǜṲṳṺṻẄẅẌẍŸÿ"
 
         tests = [ ('.suffix', 'prefix'), # simple
-                  ("\t", "\t"), # tab
-                  ("~`!@#$%^&*()-_+=", "x"), # non-alphanumeric characters
-                  ("[]{};':,.?<>\|", "x"), # non-alphanumeric characters
                   ("x".zfill(100), ".y".zfill(100)), # long
                   (" ", " "), # space
-                  ('"', "'"), # quotation marks
+                  ("'", "'"), # quotation mark
+                  ("~`!@#$%^&()-_+=", "x"), # non-alphanumeric characters
+                  ("[]{};',.", "x"), # non-alphanumeric characters
                   (u'a', u'a'), # simple unicode
                   (hebrew, hebrew),
                   (katanana, katanana),
                   (umlauts, umlauts) ]
+        if 'linux2' == sys.platform:
+            # Windows doesn't allow these characters
+            tests.append( ('"', '*') )
+            tests.append( ('\t', '\\') )
+            tests.append( (':?', '<>|') )
         for test in tests:
-            for shred in [True, False]:
-                (fd, filename) = tempfile.mkstemp(test[0], 'bleachbit-test' + test[1])
-                self.assert_(os.path.exists(filename))
-                for x in range(0, 4096/5):
-                    bytes = os.write(fd, "top secret")
-                    self.assertEqual(bytes, 10)
-                os.close(fd)
-                self.assert_(os.path.exists(filename))
-                delete(filename, shred)
-                self.assert_(not os.path.exists(filename))
+            (fd, filename) = tempfile.mkstemp(test[0], 'bleachbit-test' + test[1])
+            self.assert_(os.path.exists(filename))
+            for x in range(0, 4096):
+                bytes_written = os.write(fd, "top secret")
+                self.assertEqual(bytes_written, 10)
+            os.close(fd)
+            self.assert_(os.path.exists(filename))
+            delete(filename, shred)
+            self.assert_(not os.path.exists(filename))
+
+        if sys.platform == 'win32':
+            return
+
+        # test file with mode 0444/-r--r--r--
+        (fd, filename) = tempfile.mkstemp()
+        os.close(fd)
+        os.chmod(filename, 0444)
+        delete(filename, shred)
+        self.assert_(not os.path.exists(filename))
 
         # test symlink
         (fd, filename) = tempfile.mkstemp()
@@ -332,21 +494,21 @@ class TestFileUtilities(unittest.TestCase):
         self.assert_(os.path.exists(filename))
         linkname = '/tmp/bleachbitsymlinktest'
         if os.path.lexists(linkname):
-            delete(linkname)
+            delete(linkname, shred)
         self.assert_(not os.path.lexists(linkname))
         os.symlink(filename, linkname)
         self.assert_(os.path.lexists(linkname))
-        delete(linkname)
+        delete(linkname, shred)
         self.assert_(os.path.exists(filename))
         self.assert_(not os.path.lexists(linkname))
-        delete(filename)
+        delete(filename, shred)
         self.assert_(not os.path.exists(filename))
 
         # test broken symlink
         os.symlink(filename, linkname)
         self.assert_(os.path.lexists(linkname))
         self.assert_(not os.path.exists(linkname))
-        delete(linkname)
+        delete(linkname, shred)
         self.assert_(not os.path.exists(linkname))
 
         # test fifo
@@ -354,34 +516,64 @@ class TestFileUtilities(unittest.TestCase):
         ret = subprocess.call(args)
         self.assertEqual(ret, 0)
         self.assert_(os.path.exists(filename))
-        delete(filename)
+        delete(filename, shred)
         self.assert_(not os.path.exists(filename))
 
 
+    def test_ego_owner(self):
+        """Unit test for ego_owner()"""
+        if 'win32' == sys.platform:
+            return
+        self.assertEqual(ego_owner('/bin/ls'), os.getuid() == 0)
+
+
+    def test_exists_in_path(self):
+        """Unit test for exists_in_path()"""
+        filename = 'ls'
+        if sys.platform == 'win32':
+            filename = 'cmd.exe'
+        self.assert_(exists_in_path(filename))
+
 
     def test_exe_exists(self):
-        """Unit test for method exe_exists()"""
+        """Unit test for exe_exists()"""
         tests = [ ("/bin/sh", True), \
             ("sh", True), \
             ("doesnotexist", False), \
             ("/bin/doesnotexist", False) ]
+        if sys.platform == 'win32':
+            tests = [ ('c:\\windows\\system32\\cmd.exe', True), \
+                      ('cmd.exe', True),
+                      ('doesnotexist', False),
+                      ('c:\\windows\\doesnotexist.exe', False) ]
         for test in tests:
             self.assertEqual(exe_exists(test[0]), test[1])
 
 
+    def test_expand_glob_join(self):
+        """Unit test for expand_glob_join()"""
+        if sys.platform == 'linux2':
+            expand_glob_join('/bin', '*sh')
+        if sys.platform == 'win32':
+            expand_glob_join('c:\windows', '*.exe')
+
+
     def test_getsize(self):
         """Unit test for method getsize()"""
-        import tempfile
-
         # create regular file
         (handle, filename) = tempfile.mkstemp("regulartest")
         os.write(handle, "abcdefghij" * 12345)
         os.close(handle)
+
+        if sys.platform == 'win32':
+            self.assertEqual(getsize(filename), 10 * 12345)
+            return
+
         output = subprocess.Popen(["du", "-h", filename], stdout=subprocess.PIPE).communicate()[0]
         output = output.replace("\n", "")
         du_size = output.split("\t")[0] + "B"
         print "output = '%s', size='%s'" % (output, du_size)
-        du_bytes = self.__human_to_bytes(du_size)
+        du_bytes = human_to_bytes(du_size)
         print output, du_size, du_bytes
         self.assertEqual(getsize(filename), du_bytes)
 
@@ -399,6 +591,57 @@ class TestFileUtilities(unittest.TestCase):
         delete(filename)
 
 
+    def test_getsizedir(self):
+        """Unit test for getsizedir()"""
+        path = '/bin'
+        if sys.platform == 'win32':
+            path = 'c:\\windows\\system32'
+        self.assert_(getsizedir(path) > 0)
+
+
+
+    def test_globex(self):
+        """Unit test for method globex()"""
+        for path in globex('/bin/*', '/ls$'):
+            self.assertEqual(path, '/bin/ls')
+
+
+    def test_listdir(self):
+        """Unit test for listdir()"""
+        for pathname in listdir(('/tmp','~/.config/')):
+            self.assert_(os.path.lexists(pathname), \
+                "does not exist: %s" % pathname)
+
+
+    def test_wipe_contents(self):
+        """Unit test for wipe_delete()"""
+
+        # create test file
+        (handle, filename) = tempfile.mkstemp("wipetest")
+        os.write(handle, "abcdefghij" * 12345)
+        os.close(handle)
+
+        # wipe it
+        wipe_contents(filename)
+
+        # check it
+        f = open(filename, 'rb')
+        while True:
+            byte = f.read(1)
+            if "" == byte:
+                break
+            self.assertEqual(byte, chr(0))
+        f.close()
+
+        # clean up
+        os.remove(filename)
+
+
+    def test_wipe_path(self):
+        """Unit test for wipe_path()"""
+        pathname = tempfile.gettempdir()
+        wipe_path(pathname)
+
 
     def test_vacuum_sqlite3(self):
         """Unit test for method vacuum_sqlite3()"""
@@ -406,7 +649,6 @@ class TestFileUtilities(unittest.TestCase):
         try:
             import sqlite3
         except ImportError, e:
-            import sys
             if sys.version_info[0] == 2 and sys.version_info[1] < 5:
                 print "Warning: Skipping test_vacuum_sqlite3() on old Python"
                 return
@@ -439,7 +681,8 @@ class TestFileUtilities(unittest.TestCase):
 
     def test_OpenFiles(self):
         """Unit test for class OpenFiles"""
-        import tempfile
+        if 'win32' == sys.platform:
+            return
 
         (handle, filename) = tempfile.mkstemp()
         self.assertEqual(openfiles.is_open(filename), True)
@@ -455,5 +698,13 @@ class TestFileUtilities(unittest.TestCase):
 
 
 if __name__ == '__main__':
+    if len(sys.argv) == 2 and sys.argv[1] == 'full':
+        print 'info: Running all tests.'
+        del sys.argv[1]
+        print sys.argv
+    else:
+        print "warning: Skipping some tests.  Use argument 'full' to run them."
+        del TestFileUtilities.test_wipe_path
     unittest.main()
 
+
