@@ -27,9 +27,13 @@ Integration specific to Unix-like operating systems
 import glob
 import os
 import re
+import shlex
+import subprocess
+import sys
 import ConfigParser
 
 import FileUtilities
+import General
 
 HAVE_GNOME_VFS = True
 try:
@@ -44,17 +48,50 @@ except:
         gnomevfs = gnome.vfs
 
 
+def free_space(pathname):
+    """Return free space in bytes"""
+    args = ['df', '-B', '1', pathname]
+    (rc, stdout, stderr) = General.run_external(args)
+    if 0 != rc:
+        return None
+    line = stdout.split('\n')[1]
+    match = re.search("^[/a-z0-9]+\s+[0-9]+\s+[0-9]+([0-9]+)", line)
+    if not match:
+        return None
+    return int(match.groups(0)[0])
+
+
+def guess_overwrite_paths():
+    """Guess which partitions to overwrite (to hide deleted files)"""
+    home = os.path.expanduser("~")
+    if free_space(home) == free_space('/tmp/'):
+        return [ home ]
+    else:
+        return [ home, '/tmp' ]
+
 
 def locale_to_language(locale):
     """Convert the locale code to a language code (generally ISO 639)"""
     if 'klingon' == locale:
         return locale
-    matches = re.findall("^([a-z]{2,3})([_-][a-zA-Z]{2,4})?(\.[a-zA-Z0-9-]*)?(@[a-zA-Z]*)?$", locale)
+    pattern = "^([a-z]{2,3})([_-][a-zA-Z]{2,4})?(\.[a-zA-Z0-9-]*)?(@[a-zA-Z]*)?$"
+    matches = re.findall(pattern, locale)
     if 1 > len(matches):
         raise ValueError("Invalid locale_code '%s'" % (locale,))
     return matches[0][0]
 
 
+def locale_globex(globpath, regex):
+    """List a path by glob, filter by regex, and return tuple
+    in format (locale, pathname)"""
+    for pathname in FileUtilities.globex(globpath, regex):
+        match = re.search(regex, pathname)
+        if None == match:
+            continue
+        locale_code = match.groups(0)[0]
+        yield (locale_code, pathname)
+
+
 class Locales:
     """Find languages and localization files"""
 
@@ -95,6 +132,8 @@ class Locales:
     __basedirs = [ os.path.expanduser('~/.local/share/locale/'),
                 '/usr/local/share/locale/',
                 '/usr/share/apps/ksgmltools2/customization/',
+                '/usr/share/calendar', # Ubuntu 8.10
+                '/usr/share/cups/locale', # Ubuntu 8.10
                 '/usr/share/doc/kde/HTML/',
                 '/usr/share/doc/HTML/release-notes/', # Fedora 10
                 '/usr/share/doc/thunar-data/html',
@@ -105,6 +144,11 @@ class Locales:
 
     __ignore = ['all_languages', 'C', 'l10n', 'locale.alias', 'default']
 
+    ooosharedirs = [ '/usr/lib/openoffice/share/', # Ubuntu 8
+        '/usr/lib/openoffice.org/basis3.0/share/', # Fedora 10
+        '/opt/ooo-dev/basis3.0/share', # Sun development snapshot
+        '/opt/openoffice.org/basis3.0/share/' ] # Sun
+
 
     def __init__(self):
         self.__languages = []
@@ -115,13 +159,7 @@ class Locales:
                 dirname = os.path.join('/usr/share/gnome/help', gapp)
                 self.__basedirs.append(dirname)
 
-
-        ooosharedirs = [ '/usr/lib/openoffice/share/', # Ubuntu 8
-            '/usr/lib/openoffice.org/basis3.0/share/', # Fedora 10
-            '/opt/ooo-dev/basis3.0/share', # Sun development snapshot
-            '/opt/openoffice.org/basis3.0/share/' ] # Sun
-
-        for ooosharedir in ooosharedirs:
+        for ooosharedir in self.ooosharedirs:
             suffixes = [ 'autotext/',
                 'config/soffice.cfg/global/accelerator/',
                 'registry/res/',
@@ -156,17 +194,20 @@ class Locales:
             try:
                 lang = locale_to_language(locale)
             except:
-                print "Warning: invalid path '%s' where expecting a locale" % locale
                 continue
             if not lang in self.__languages:
                 self.__languages.append(lang)
-        self.__languages = sorted(self.__languages)
+        import Options
+        selected_languages = Options.options.get_languages()
+        if None != selected_languages:
+            self.__languages += selected_languages
+        self.__languages = sorted(set(self.__languages))
         self.__scanned = True
 
 
     def iterate_languages(self):
         """Return each language code (generally ISO 639)"""
-        if False == self.__scanned:
+        if not self.__scanned:
             self.__scan()
         for lang in self.__languages:
             yield lang
@@ -174,6 +215,11 @@ class Locales:
 
     def __localization_path(self, basedir, language_filter, dir_filter):
         """Return localization paths in a single directory tree"""
+        if -1 != basedir.find('*'):
+            for basedir2 in glob.iglob(basedir):
+                for path in self.__localization_path(basedir2, language_filter, dir_filter):
+                    yield path
+            return
         if not os.path.exists(basedir):
             return
         for path in os.listdir(basedir):
@@ -183,8 +229,7 @@ class Locales:
             try:
                 language_code = locale_to_language(path)
             except:
-                 print "Warning: invalid path '%s' where expecting a locale" % path
-                 continue
+                continue
             if None != language_filter and language_filter(locale_code, language_code):
                 continue
             locale_dirname = os.path.join(basedir, locale_code)
@@ -196,72 +241,95 @@ class Locales:
     def localization_paths(self, language_filter):
         """Return paths containing localization files"""
 
-        # general
+        ##
+        ## general
+        ##
         for basedir in self.__basedirs:
             dir_filter = lambda d: d in self.__ignore
             for path in self.__localization_path(basedir, language_filter, dir_filter):
                 yield path
 
-        # CUPS
-        # example: /usr/share/cups/templates/es/jobs.tmpl
-        dir_filter = lambda d: d.endswith('tmpl')
-        for path in self.__localization_path('/usr/share/cups/templates/', language_filter, dir_filter):
-            yield path
-        # Fedora 10
-        #   example: /usr/share/cups/www/es/images/button-add-printer.gif
-        dir_filter = lambda d: d in ['cups.css', 'cups-printable.css', 'favicon.ico', 'help', 'images', 'index.html', 'robots.txt']
-        for path in self.__localization_path('/usr/share/cups/www/', language_filter, dir_filter):
-            yield path
-        # Ubuntu 8.10
-        #   example: /usr/share/cups/doc-root/es/images/button-add-printer.gif
-        for path in self.__localization_path('/usr/share/cups/doc-root/', language_filter, dir_filter):
-            yield path
+        ###
+        ### the locale is the directory name
+        ###
+
+        lps = []
+
+        # CUPS, Fedora 10
+        # /usr/share/cups/templates/es/jobs.tmpl
+        lps += ( ('/usr/share/cups/templates/', lambda d: d.endswith('tmpl')) , )
+
+        # CUPS, Fedora 10
+        # /usr/share/cups/www/es/images/button-add-printer.gif
+        dir_filter = lambda d: d in ['cups.css', 'cups-printable.css', \
+            'favicon.ico', 'help', 'images', 'index.html', 'robots.txt']
+        lps += ( ('/usr/share/cups/www/', dir_filter) , )
+
+        # CUPS, Ubuntu 8.10
+        # /usr/share/cups/doc-root/es/images/button-add-printer.gif
+        lps += ( ('/usr/share/cups/doc-root/', dir_filter) , )
+
+        # Evolution
+        # /usr/share/evolution/2.22/help/quickref/es/quickref.pdf
+        lps += ( ('/usr/share/evolution/*/help/quickref/', dir_filter) , )
 
         # foomatic
         dir_filter = lambda d: 2 != len(d)
-        for path in self.__localization_path('/usr/share/foomatic/db/source/PPD/Kyocera/', language_filter, dir_filter):
-            yield path
-
-        # glibc-common
-        # example: /usr/share/i18n/locales/es_ES@euro
-        for path in glob.iglob('/usr/share/i18n/locales/*_*'):
-            locale_code = os.path.basename(path)
-            if locale_code.startswith('iso14') or locale_code.startswith('translit'):
-                continue
-            language_code = locale_to_language(locale_code)
-            if None != language_filter and language_filter(locale_code, language_code):
-                continue
-            yield path
+        lps += ( ('/usr/share/foomatic/db/source/PPD/Kyocera/', dir_filter) , )
 
         # man pages
-        # example: /usr/share/man/es/man1/man.1.gz
+        # /usr/share/man/es/man1/man.1.gz
         dir_filter = lambda d: d.startswith('man')
-        for path in self.__localization_path('/usr/share/man/', language_filter, dir_filter):
-            yield path
+        lps += ( ('/usr/share/man/', dir_filter) , )
 
-        # OMF
-        # example: /usr/share/omf/gedit/gedit-es.omf
-        for path in glob.iglob('/usr/share/omf/*/*-*.omf'):
-            locale_code = path[path.rfind("-") + 1 : path.rfind(".") ]
-            if 'C' == locale_code:
-                continue
-            try:
+        # process lps
+        for lp in lps:
+            for path in self.__localization_path(lp[0], language_filter, lp[1]):
+                yield path
+
+
+        ###
+        ### locale_globex
+        ###
+        globexs = []
+
+        # /usr/share/i18n/locales/es_ES@euro
+        globexs += ( ('/usr/share/i18n/locales/??_*', 'locales/([a-z]{2}_[A-Z]{2})'), )
+        # /usr/share/ppd/splix/samsung/ml1740fr.ppd
+        globexs += ( ('/usr/share/ppd/splix/*/*fr.ppd', '(fr).ppd$' ), )
+        # /usr/share/espeak-data/es_dict
+        globexs += ( ('/usr/share/espeak-data/*_dict', '/([a-z]{2,3})_dict$' ), )
+        # /usr/share/espeak-data/voices/en/en-n
+        globexs += ( ('/usr/share/espeak-data/voices/en/*', '/(en)-?[a-z]{0,2}$' ), )
+        # /usr/share/espeak-data/voices/es-la
+        globexs += ( ('/usr/share/espeak-data/voices/*', '/([a-z]{2,3}(-[a-z]{2})?)$' ), )
+        # /usr/share/espeak-data/voices/mb/mb-ro1-en, do not match mb-us*
+        globexs += ( ('/usr/share/espeak-data/voices/mb/mb-*', '/mb-([a-tv-z][a-z])[0-9](-[a-z]{2,3})?$' ), )
+        # /usr/share/hplip/data/localization/hplip_es.qm
+        globexs += ( ('/usr/share/hplip/data/localization/hplip_??.qm', '_([a-z]{2}).qm$' ), )
+        # /usr/share/myspell/dicts/hyph_es_ES.dic
+        globexs += ( ('/usr/share/myspell/dicts/hyph_??_??.dic', '([a-z]{2}_[A-Z]{2}).dic$' ), )
+        # /usr/share/omf/gedit/gedit-es.omf
+        globexs += ( ('/usr/share/omf/*/*-*.omf', '-([a-z]{2}).omf$'), )
+        # OpenOffice.org
+        # /usr/lib/openoffice/share/autocorr/acor_es-ES.dat
+        for ooosharedir in self.ooosharedirs:
+            globexs += ( (ooosharedir + '/autocorr/acor_*.dat', 'acor_([a-z]{2}(-[A-Z]{2})?).dat$'), )
+        # TCL on Fedora 10a
+        # /usr/share/tcl8.5/msgs/es.msg
+        # /usr/share/tcl8.5/msgs/es_mx.msg
+        globexs += ( ('/usr/share/tcl*/msgs/?*.msg', '/([a-z]{2}(_[a-z]{2})?).msg$'), )
+
+        # process reglobs
+        for (globpath, regex) in globexs:
+            for (locale_code, path) in locale_globex(globpath, regex):
                 language_code = locale_to_language(locale_code)
-            except:
-                print "Warning: OMF path '%s' does not look like a locale" % path
-                continue
-            if None != language_filter and language_filter(locale_code, language_code):
-                continue
-            yield path
+                if 'mb' == language_code: # not a real code but found in espeak Ubuntu 9.04
+                    continue
+                if None != language_filter and language_filter(locale_code, language_code):
+                    continue
+                yield path
 
-        # TCL
-        # example: /usr/share/tcl8.5/msgs/es_mx.msg
-        for path in glob.glob('/usr/share/tcl*/msgs/*.msg'): 
-            locale_code = os.path.splitext(os.path.basename(path))[0]
-            language_code = locale_to_language(locale_code)
-            if None != language_filter and language_filter(locale_code, language_code):
-                continue
-            yield path
 
 
     def native_name(self, language_code):
@@ -281,6 +349,66 @@ class Locales:
         return None
 
 
+def apt_autoclean():
+    """Run 'apt-get autoclean' and return the size (un-rounded, in bytes)
+        of freed space"""
+
+    if not FileUtilities.exe_exists('apt-get'):
+        raise RuntimeError(_('Executable not found: %s') % 'apt-get')
+
+    args = ['apt-get', 'autoclean']
+
+    process = subprocess.Popen(args, \
+        stderr =  subprocess.STDOUT, stdout = subprocess.PIPE)
+
+    total_bytes = 0
+
+    while True:
+        line = process.stdout.readline().replace("\n", "")
+        if line.startswith('E: '):
+            raise RuntimeError(line)
+        # Del cups-common 1.3.9-17ubuntu3 [1165kB]
+        match = re.search("^Del .*\[([0-9.]+[a-zA-Z]{2})\]", line)
+        if match:
+            pkg_bytes_str = match.groups(0)[0]
+            pkg_bytes = FileUtilities.human_to_bytes(pkg_bytes_str.upper())
+            total_bytes += pkg_bytes
+        if "" == line and process.poll() != None:
+            break
+
+    return total_bytes
+
+
+def apt_autoremove():
+    """Run 'apt-get auturemove' and return the size (un-rounded, in bytes)
+        of freed space"""
+
+    if not FileUtilities.exe_exists('apt-get'):
+        raise RuntimeError(_('Executable not found: %s') % 'apt-get')
+
+    args = ['apt-get', '--yes', 'autoremove']
+
+    process = subprocess.Popen(args, \
+        stderr =  subprocess.STDOUT, stdout = subprocess.PIPE)
+
+    total_bytes = 0
+
+    while True:
+        line = process.stdout.readline().replace("\n", "")
+        if line.startswith('E: '):
+            raise RuntimeError(line)
+        # After this operation, 74.7MB disk space will be freed.
+        match = re.search(", ([0-9.]+[a-zA-Z]{2}) disk space will be freed", line)
+        if match:
+            pkg_bytes_str = match.groups(0)[0]
+            pkg_bytes = FileUtilities.human_to_bytes(pkg_bytes_str.upper())
+            total_bytes += pkg_bytes
+        if "" == line and process.poll() != None:
+            break
+
+    return total_bytes
+
+
 def __is_broken_xdg_desktop_application(config, desktop_pathname):
     """Returns boolean whether application deskop entry file is broken"""
     if not config.has_option('Desktop Entry', 'Exec'):
@@ -358,6 +486,47 @@ def is_broken_xdg_desktop(pathname):
     return False
 
 
+def is_running(exename):
+    """Check whether exename is running"""
+    for filename in glob.iglob("/proc/*/exe"):
+        try:
+            target = os.path.realpath(filename)
+        except TypeError:
+            # happens, for example, when link points to
+            # '/etc/password\x00 (deleted)'
+            continue
+        except OSError:
+            # 13 = permission denied
+            continue
+        if exename == os.path.basename(target):
+            return True
+    return False
+
+
+def rotated_logs():
+    """Yield a list of rotated (i.e., old) logs in /var/log/"""
+    # Ubuntu 9.04
+    # /var/log/dmesg.0
+    # /var/log/dmesg.1.gz
+    # Fedora 10
+    # /var/log/messages-20090118
+    globpaths = ( '/var/log/*.[0-9]', \
+         '/var/log/*/*.[0-9]', \
+        '/var/log/*.gz', \
+        '/var/log/*/*gz', \
+        '/var/log/*/*.old', \
+        '/var/log/*.old' )
+    for globpath in globpaths:
+        for path in glob.iglob(globpath):
+            yield path
+    regex = '-[0-9]{8}$'
+    globpaths = ( '/var/log/*-*', '/var/log/*/*-*' )
+    for path in FileUtilities.globex(globpaths, regex):
+        whitelist_re = '^/var/log/(removed_)?(packages|scripts)'
+        if None == re.match(whitelist_re, path): # for Slackware, Launchpad #367575
+            yield path
+
+
 def wine_to_linux_path(wineprefix, windows_pathname):
     """Return a Linux pathname from an absolute Windows pathname and Wine prefix"""
     drive_letter = windows_pathname[0]
@@ -367,6 +536,31 @@ def wine_to_linux_path(wineprefix, windows_pathname):
     return os.path.join(wineprefix, windows_pathname)
 
 
+def yum_clean():
+    """Run 'yum clean all' and return size in bytes recovered"""
+    if os.path.exists('/var/run/yum.pid'):
+        msg = _("%s cannot be cleaned because it is currently running.  Close it, and try again.") % "Yum"
+        raise RuntimeError(msg)
+    if not FileUtilities.exe_exists('yum'):
+        raise RuntimeError(_('Executable not found: %s') % 'yum')
+    old_size = FileUtilities.getsizedir('/var/cache/yum')
+    args  = ['yum', "--enablerepo=*", 'clean', 'all']
+    p = subprocess.Popen(args, stderr=subprocess.STDOUT, \
+        stdout=subprocess.PIPE)
+    while True:
+        line = p.stdout.readline().replace("\n", "")
+        if -1 != line.find('You need to be root'):
+            raise RuntimeError(line)
+        if -1 != line.find('Another app is currently holding'):
+            print "debug: yum: '%s'" % line
+            old_size = FileUtilities.getsizedir('/var/cache/yum')
+        if "" == line and p.poll() != None:
+            break
+    print 'debug: yum process return code = %d' % p.returncode
+    new_size = FileUtilities.getsizedir('/var/cache/yum')
+    return old_size - new_size
+
+
 locales = Locales()
 
 
@@ -380,6 +574,30 @@ class TestUnix(unittest.TestCase):
         self.locales = Locales()
 
 
+    def test_apt_autoclean(self):
+        """Unit test for method apt_autoclean()"""
+        if 0 != os.geteuid() or not FileUtilities.exe_exists('apt-get'):
+            self.assertRaises(RuntimeError, apt_autoclean)
+        else:
+            bytes_freed = apt_autoclean()
+            self.assert_(type(bytes_freed) is int)
+
+
+    def test_free_space(self):
+        """Unit test for free_space()"""
+        home = os.path.expanduser("~")
+        result = free_space(home)
+        self.assertNotEqual(result, None)
+        self.assert_(result > -1)
+        self.assert_(type(result) is int)
+
+
+    def test_guess_overwrite_paths(self):
+        """Unit test for guess_overwrite_paths()"""
+        for path in guess_overwrite_paths():
+            self.assert_(os.path.isdir(path))
+
+
     def test_is_broken_xdg_desktop(self):
         """Unit test for is_broken_xdg_desktop()"""
         menu_dirs = [ '/usr/share/applications', \
@@ -395,16 +613,21 @@ class TestUnix(unittest.TestCase):
                 self.assert_(type(is_broken_xdg_desktop(filename) is bool))
 
 
-    def iterate_languages(self):
+    def test_is_running(self):
+        exe = os.path.basename(sys.executable)
+        self.assertEqual(True, is_running(exe))
+
+
+    def test_iterate_languages(self):
         """Unit test for the method iterate_languages()"""
         for language in self.locales.iterate_languages():
             self.assert_(type(language) is str)
-            self.assert_(len(str) > 0)
-            self.assert_(len(str) < 9)
+            self.assert_(len(language) > 0)
+            self.assert_(len(language) < 9)
 
 
     def test_locale_to_language(self):
-        """Test method locale_to_language()"""
+        """Unit test for locale_to_language()"""
         tests = [ ('en', 'en'),
             ('en_US', 'en'),
             ('en_US@piglatin', 'en'),
@@ -420,21 +643,48 @@ class TestUnix(unittest.TestCase):
         self.assertRaises(ValueError, locale_to_language, 'C')
         self.assertRaises(ValueError, locale_to_language, 'English')
 
+    def test_locale_globex(self):
+        """Unit test for locale_globex"""
+
+        locale = locale_globex('/bin/ls', '(ls)$').next()
+        self.assertEqual(locale, ('ls', '/bin/ls'))
+
+        fakepath = '/usr/share/omf/gedit/gedit-es.omf'
+
+        def test_yield(pathname, regex):
+            """Replacement for globex()"""
+            yield fakepath
+
+        old_globex = FileUtilities.globex
+        FileUtilities.globex = test_yield
+
+        func = locale_globex('/usr/share/omf/*/*-*.omf', '-([a-z]{2}).omf$')
+        actual = func.next()
+        expect = ('es', fakepath)
+        self.assertEqual(actual, expect, "Expected '%s' but got '%s'" % (expect, actual))
+        FileUtilities.globex = old_globex
 
     def test_localization_paths(self):
-        """Test method localization_paths()"""
+        """Unit test for localization_paths()"""
         for path in locales.localization_paths(lambda x, y: False):
             self.assert_(os.path.lexists(path))
 
 
     def test_native_name(self):
-        """Test method native_name()"""
+        """Unit test for native_name()"""
         tests = [ ('en', 'English'),
             ('es', 'Español') ]
         for test in tests:
             self.assertEqual(self.locales.native_name(test[0]), test[1])
 
 
+    def test_rotated_logs(self):
+        """Unit test for rotated_logs()"""
+        for path in rotated_logs():
+            self.assert_(os.path.exists(path), \
+                "Rotated log path '%s' does not exist" % path)
+
+
     def test_wine_to_linux_path(self):
         """Unit test for wine_to_linux_path()"""
         tests = [ ("/home/foo/.wine", \
@@ -443,7 +693,17 @@ class TestUnix(unittest.TestCase):
         for test in tests:
             self.assertEqual(wine_to_linux_path(test[0], test[1]), test[2])
 
+    def test_yum_clean(self):
+        """Unit test for yum_clean()"""
+        if 0 != os.geteuid() or os.path.exists('/var/run/yum.pid') \
+            or not FileUtilities.exe_exists('yum'):
+            self.assertRaises(RuntimeError, yum_clean)
+        else:
+            bytes_freed = yum_clean()
+            self.assert_(type(bytes_freed) is int)
+            print 'debug: yum bytes cleaned %d', bytes_freed
 
-if __name__ == '__main__':
+
+if __name__ == '__main__' and 'linux2' == sys.platform:
     unittest.main()
 
