@@ -24,22 +24,32 @@ Check for updates via the Internet
 import platform
 import socket
 import sys
+import traceback
 import urllib2
 import xml.dom.minidom
 
-import globals
-from globals import APP_VERSION, socket_timeout
+import Common
 
 
 def user_agent():
     """Return the user agent string"""
-    ver = APP_VERSION
-    __platform = platform.system()
-    __os = platform.uname()[2]
-    cpu = platform.machine()
-    import locale
-    __locale = locale.getdefaultlocale()[0]
-    agent = "BleachBit/%s (%s; %s, %s; %s)" % (ver, __platform, __os, cpu, __locale)
+    __platform = platform.system() # Linux or Windows
+    __os = platform.uname()[2] # e.g., 2.6.28-12-generic or XP
+    if sys.platform == "win32":
+        # Python 2.5.4 shows uname()[2) as Vista on Windows 7
+        __os = platform.uname()[3][0:3] # 5.1 = Windows XP, 6.0 = Vista, 6.1 = 7
+    if sys.platform == 'linux2':
+        dist = platform.dist()
+        __os = dist[0] + '/' + dist[1] + '-' + dist[2]
+    __locale = ""
+    try:
+        import locale
+        __locale = locale.getdefaultlocale()[0] # e.g., en_US
+    except:
+        traceback.print_exc()
+
+    agent = "BleachBit/%s (%s; %s; %s)" % (Common.APP_VERSION, \
+        __platform, __os, __locale)
     return agent
 
 
@@ -61,9 +71,9 @@ class Update:
         """Return boolean whether update is available"""
         opener = urllib2.build_opener()
         opener.addheaders = [('User-Agent', user_agent())]
-        socket.setdefaulttimeout(socket_timeout)
+        socket.setdefaulttimeout(Common.socket_timeout)
         try:
-            handle = opener.open(globals.update_check_url)
+            handle = opener.open(Common.update_check_url)
             dom = xml.dom.minidom.parse(handle)
         except:
             print _("Error when checking for updates: "), str(sys.exc_info()[1])
@@ -91,7 +101,7 @@ class TestUpdate(unittest.TestCase):
         self.assert_ (type(available) is bool)
 
         # test failure
-        globals.update_check_url = "http://www.surelydoesnotexist.com/foo"
+        Common.update_check_url = "http://www.surelydoesnotexist.com/foo"
         self.assertEqual(update.is_update_available(), False)
 
 
